@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 
 from trytond.model import Workflow, ModelView
@@ -7,10 +7,9 @@ from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 
 __all__ = ['Move']
-__metaclass__ = PoolMeta
 
 
-class Move:
+class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
 
     def _get_account_stock_move_lines(self, type_):
@@ -24,8 +23,7 @@ class Move:
         assert type_.startswith('in_') or type_.startswith('out_'), \
             'wrong type'
 
-        with Transaction().set_user(0, set_context=True):
-            move_line = AccountMoveLine()
+        move_line = AccountMoveLine()
         if ((type_.endswith('supplier')
                     or type_ == 'in_production')
                 and self.product.cost_price_method != 'fixed'):
@@ -33,8 +31,9 @@ class Move:
                 unit_price = Currency.compute(self.currency, self.unit_price,
                     self.company.currency, round=False)
         else:
-            unit_price = Uom.compute_price(self.product.default_uom,
-                self.cost_price, self.uom)
+            unit_price = self.cost_price
+        unit_price = Uom.compute_price(self.product.default_uom,
+            unit_price, self.uom)
         amount = self.company.currency.round(
                 Decimal(str(self.quantity)) * unit_price)
 
@@ -58,10 +57,9 @@ class Move:
         '''
         pool = Pool()
         AccountMoveLine = pool.get('account.move.line')
-        with Transaction().set_user(0, set_context=True):
-            move_line = AccountMoveLine(
-                account=self.product.account_stock_used,
-                )
+        move_line = AccountMoveLine(
+            account=self.product.account_stock_used,
+            )
         if not amount:
             return
         if amount >= Decimal('0.0'):
@@ -77,13 +75,13 @@ class Move:
         Get account move type
         '''
         type_ = (self.from_location.type, self.to_location.type)
-        if type_ == ('supplier', 'storage'):
+        if type_ in [('supplier', 'storage'), ('supplier', 'drop')]:
             return 'in_supplier'
-        elif type_ == ('storage', 'supplier'):
+        elif type_ in [('storage', 'supplier'), ('drop', 'supplier')]:
             return 'out_supplier'
-        elif type_ == ('storage', 'customer'):
+        elif type_ in [('storage', 'customer'), ('drop', 'customer')]:
             return 'out_customer'
-        elif type_ == ('customer', 'storage'):
+        elif type_ in [('customer', 'storage'), ('customer', 'drop')]:
             return 'in_customer'
         elif type_ == ('storage', 'lost_found'):
             return 'out_lost_found'
@@ -110,21 +108,29 @@ class Move:
 
         if self.product.type != 'goods':
             return
+
+        date = Date.today()
+        period_id = Period.find(self.company.id, date=date)
+        period = Period(period_id)
+        if not period.fiscalyear.account_stock_method:
+            return
+
         type_ = self._get_account_stock_move_type()
         if not type_:
             return
-        if type_ == 'supplier_customer':
-            account_move_lines = self._get_account_stock_move_lines(
-                'in_supplier')
-            account_move_lines.extend(self._get_account_stock_move_lines(
-                    'out_customer'))
-        elif type_ == 'customer_supplier':
-            account_move_lines = self._get_account_stock_move_lines(
-                'in_customer')
-            account_move_lines.extend(self._get_account_stock_move_lines(
-                    'out_supplier'))
-        else:
-            account_move_lines = self._get_account_stock_move_lines(type_)
+        with Transaction().set_context(date=date):
+            if type_ == 'supplier_customer':
+                account_move_lines = self._get_account_stock_move_lines(
+                    'in_supplier')
+                account_move_lines.extend(self._get_account_stock_move_lines(
+                        'out_customer'))
+            elif type_ == 'customer_supplier':
+                account_move_lines = self._get_account_stock_move_lines(
+                    'in_customer')
+                account_move_lines.extend(self._get_account_stock_move_lines(
+                        'out_supplier'))
+            else:
+                account_move_lines = self._get_account_stock_move_lines(type_)
 
         amount = Decimal('0.0')
         for line in account_move_lines:
@@ -133,25 +139,14 @@ class Move:
         if move_line:
             account_move_lines.append(move_line)
 
-        date = self.effective_date or Date.today()
-        period_id = Period.find(self.company.id, date=date)
         account_configuration = AccountConfiguration(1)
-        with Transaction().set_user(0, set_context=True):
-            return AccountMove(
-                journal=account_configuration.stock_journal,
-                period=period_id,
-                date=date,
-                origin=self,
-                lines=account_move_lines,
-                )
-
-    @classmethod
-    def copy(cls, moves, default=None):
-        if default is None:
-            default = {}
-        default = default.copy()
-        default.setdefault('account_move', None)
-        return super(Move, cls).copy(moves, default=default)
+        return AccountMove(
+            journal=account_configuration.stock_journal,
+            period=period_id,
+            date=date,
+            origin=self,
+            lines=account_move_lines,
+            )
 
     @classmethod
     @ModelView.button
@@ -162,8 +157,8 @@ class Move:
         super(Move, cls).do(moves)
         account_moves = []
         for move in moves:
-            account_moves.append(move._get_account_stock_move())
-        with Transaction().set_user(0, set_context=True):
-            account_moves = AccountMove.create(
-                [m._save_values for m in account_moves if m])
-            AccountMove.post(account_moves)
+            account_move = move._get_account_stock_move()
+            if account_move:
+                account_moves.append(account_move)
+        AccountMove.save(account_moves)
+        AccountMove.post(account_moves)
